<!DOCTYPE html>
<html>
<head>
    <title>{% block html_title %}{% endblock %}</title>
    {% block meta %}{% endblock %}
</head>
<body>
    <h1>{% block page_title %}{% endblock %}</h1>

    {% block content %}{% endblock %}
    {% block footer %}
        <hr>
        <p><a href="javascript:history.back()">Back</a></p>
        <p><a href="{% url home_view %}">Home</a></p>
    {% endblock %}
</body>
</html>