{% extends "base.html" %}

{% block html_title %}Volume list{% endblock %}

{% block page_title %}Volume list{% endblock %}

{% block content %}
        <ul>
            {% for object in object_list %}
                <li><a href="{% url volume_detail_view object.id %}">{{ object.name }}</a></li>
            {% endfor %}
        </ul>
{% endblock %}
